/*
 * linux/include/asm-arm/arch-ep93xx/ts72xx.h
 */

/*
 * TS72xx memory map:
 *
 * virt		phys		size
 * febff000	22000000	4K	TS-72xx model number register
 * febfe000	22400000	4K	options register
 * febfd000	22800000	4K	options register #2
 * febfc000	[67]0000000	4K	NAND data register
 * febfb000	[67]0400000	4K	NAND control register
 * febfa000	[67]0800000	4K	NAND busy register
 * febf9000	10800000	4K	TS-5620 RTC index register
 * febf8000	11700000	4K	TS-5620 RTC data register
 * febf7000	600ff080    	4k	TS-73xx model number register
 * febf6000	e8000808    	4k	TS-7800 RTC regs
 * xxxxxxxx	600ff408	1 byte	TS-7390 RTC index register
 * xxxxxxxx	600ff40C 	1 byte	TS-7390 RTC data register
 * xxxxxxxx	600ff870	1 byte	TS-7350/70 RTC index register
 * xxxxxxxx	600ff871	1 byte	TS-7350/70 RTC data register
 * xxxxxxxx	80840000	4k	TS-7400 RTC GPIO pins
 */

#define TS72XX_MODEL_PHYS_BASE		0x22000000
#define TS72XX_MODEL_VIRT_BASE		0xfebff000
#define TS72XX_MODEL_SIZE		0x00001000

#define TS73XX_MODEL_PHYS_BASE		0x600ff000
#define TS73XX_MODEL_VIRT_BASE		0xfebf7000
#define TS73XX_MODEL_SIZE		0x00001000

#define TS72XX_MODEL_TS7200		0x00
#define TS72XX_MODEL_TS7250		0x01
#define TS72XX_MODEL_TS7260		0x02
#define TS72XX_MODEL_TS7300		0x03
#define TS72XX_MODEL_TS7400		0x04


#define TS72XX_OPTIONS_PHYS_BASE	0x22400000
#define TS72XX_OPTIONS_VIRT_BASE	0xfebfe000
#define TS72XX_OPTIONS_SIZE		0x00001000

#define TS72XX_OPTIONS_COM2_RS485	0x02
#define TS72XX_OPTIONS_MAX197		0x01


#define TS72XX_OPTIONS2_PHYS_BASE	0x22800000
#define TS72XX_OPTIONS2_VIRT_BASE	0xfebfd000
#define TS72XX_OPTIONS2_SIZE		0x00001000

#define TS72XX_OPTIONS2_TS9420		0x04
#define TS72XX_OPTIONS2_TS9420_BOOT	0x02


#define TS72XX_NOR_PHYS_BASE		0x60000000
#define TS72XX_NOR2_PHYS_BASE		0x62000000

#define TS72XX_NAND1_DATA_PHYS_BASE	0x60000000
#define TS72XX_NAND2_DATA_PHYS_BASE	0x70000000
#define TS72XX_NAND_DATA_VIRT_BASE	0xfebfc000
#define TS72XX_NAND_DATA_SIZE		0x00001000

#define TS72XX_NAND1_CONTROL_PHYS_BASE	0x60400000
#define TS72XX_NAND2_CONTROL_PHYS_BASE	0x70400000
#define TS72XX_NAND_CONTROL_VIRT_BASE	0xfebfb000
#define TS72XX_NAND_CONTROL_SIZE	0x00001000

#define TS72XX_NAND1_BUSY_PHYS_BASE	0x60800000
#define TS72XX_NAND2_BUSY_PHYS_BASE	0x70800000
#define TS72XX_NAND_BUSY_VIRT_BASE	0xfebfa000
#define TS72XX_NAND_BUSY_SIZE		0x00001000


#define TS72XX_RTC_INDEX_VIRT_BASE	0xfebf9000
#define TS72XX_RTC_INDEX_PHYS_BASE	0x10800000
#define TS72XX_RTC_INDEX_SIZE		0x00001000

#define TS72XX_RTC_DATA_VIRT_BASE	0xfebf8000
#define TS72XX_RTC_DATA_PHYS_BASE	0x11700000
#define TS72XX_RTC_DATA_SIZE		0x00001000

#define TS7350_RTC_BASE			0x600ff000
#define TS7350_RTC_SIZE			0x00001000
#define TS7350_RTC_INDEX		0x00000870	//0x600ff870
#define TS7350_RTC_DATA			0x00000871	//0x600ff871

#define TS7390_RTC_BASE			0x600ff000
#define TS7390_RTC_SIZE			0x00001000
#define TS7390_RTC_INDEX		0x00000408	//0x600ff408
#define TS7390_RTC_DATA			0x0000040C	//0x600ff40C

#define TS7400_RTC_BASE			0x80840000
#define TS7400_RTC_SIZE			0x00001000

#define TS7350_PC104_BASE		0x600ff000
#define TS7350_PC104_SIZE		0x00000100
#define TS7350_PC104_A_LOW		0x0000004c	//0x600ff098
#define TS7350_PC104_A_UPPER		0x0000004d	//0x600ff09a
#define TS7350_PC104_B_LOW		0x0000004e	//0x600ff09c
#define TS7350_PC104_B_UPPER		0x0000004f	//0x600ff09e
#define TS7350_PC104_ISACONF		0x0000006a	//0x600ff0d4

#define TS72XX_WATCHDOG_CONTROL_VIRT_BASE  0xfebf7000
#define TS72XX_WATCHDOG_CONTROL_PHYS_BASE  0x23800000
#define TS72XX_WATCHDOG_CONTROL_SIZE  0x00001000

#define TS72XX_WATCHDOG_FEED_VIRT_BASE  0xfebf6000
#define TS72XX_WATCHDOG_FEED_PHYS_BASE  0x23c00000
#define TS72XX_WATCHDOG_FEED_SIZE  0x00001000


#ifndef __ASSEMBLY__
#include <asm/io.h>

static inline int board_is_ts7200(void)
{
	return (__raw_readb(TS72XX_MODEL_VIRT_BASE) & 0x7) == TS72XX_MODEL_TS7200;
}

static inline int board_is_ts7250(void)
{
	return (__raw_readb(TS72XX_MODEL_VIRT_BASE) & 0x7) == TS72XX_MODEL_TS7250;
}

static inline int board_is_ts7260(void)
{
	return (__raw_readb(TS72XX_MODEL_VIRT_BASE) & 0x7) == TS72XX_MODEL_TS7260;
}

static inline int board_is_ts7400(void)
{
	return (__raw_readb(TS72XX_MODEL_VIRT_BASE) & 0x7) == TS72XX_MODEL_TS7400;
}

static inline int board_is_ts7350(void)
{
	unsigned short frame_size;
	unsigned int ret = 0;
	volatile unsigned char *ax88796b = (unsigned char *)TS73XX_MODEL_VIRT_BASE + 0x800;

	//7350 and 7370 have the same board id
	if((__raw_readw(TS73XX_MODEL_VIRT_BASE + 0x80) >> 4)  == 0x6C8) {
               //7370 has an Asix chip
                frame_size = ax88796b[0x18];
                ax88796b[0x18] = 0xaa;
		__raw_readw(TS73XX_MODEL_VIRT_BASE + 0x80); ///clear bus mem

                if(ax88796b[0x18] != 0xaa) ret = 1;

                ax88796b[0x18] = frame_size;
	}

	return ret;
}

static inline int board_is_ts7370(void)
{
	unsigned char frame_size;
	unsigned int ret = 0;
	volatile unsigned char *ax88796b = (unsigned char *)TS73XX_MODEL_VIRT_BASE + 0x800;

	//7350 and 7370 have the same board id
	if((__raw_readw(TS73XX_MODEL_VIRT_BASE + 0x80) >> 4)  == 0x6C8) {
               //7370 has an Asix chip
                frame_size = ax88796b[0x18];
                ax88796b[0x18] = 0xaa;
		__raw_readw(TS73XX_MODEL_VIRT_BASE + 0x80); ///clear bus mem

                if(ax88796b[0x18] == 0xaa) ret = 1;

                ax88796b[0x18] = frame_size;
	}

	return ret;
}

static inline int board_is_ts7390(void)
{
	if((__raw_readw(TS73XX_MODEL_VIRT_BASE + 0x80) >> 4)  == 0xCDE) 
		return 1;
	else
		return 0;
}

static inline int is_max197_installed(void)
{
	return !!(__raw_readb(TS72XX_OPTIONS_VIRT_BASE) &
					TS72XX_OPTIONS_MAX197);
}

static inline int is_ts9420_installed(void)
{
	return !!(__raw_readb(TS72XX_OPTIONS2_VIRT_BASE) &
					TS72XX_OPTIONS2_TS9420);
}

#endif
