/*
 * linux/include/asm-arm/arch-ep93xx/uncompress.h
 *
 * Copyright (C) 2006 Lennert Buytenhek <buytenh@wantstofly.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 */

#include <asm/arch/ep93xx-regs.h>

static unsigned char __raw_readb(unsigned int ptr)
{
	return *((volatile unsigned char *)ptr);
}

static unsigned int __raw_readl(unsigned int ptr)
{
	return *((volatile unsigned int *)ptr);
}

static void __raw_writeb(unsigned char value, unsigned int ptr)
{
	*((volatile unsigned char *)ptr) = value;
}

static void __raw_writel(unsigned int value, unsigned int ptr)
{
	*((volatile unsigned int *)ptr) = value;
}


#define PHYS_UART1_DATA		0x808c0000
#define PHYS_UART1_FLAG		0x808c0018
#define UART1_FLAG_TXFF		0x20

static inline void putc(int c)
{
	int i;

	for (i = 0; i < 1000; i++) {
		/* Transmit fifo not full?  */
		if (!(__raw_readb(PHYS_UART1_FLAG) & UART1_FLAG_TXFF))
			break;
	}

	__raw_writeb(c, PHYS_UART1_DATA);
}

static inline void flush(void)
{
}


/*
 * Some bootloaders don't turn off DMA from the ethernet MAC before
 * jumping to linux, which means that we might end up with bits of RX
 * status and packet data scribbled over the uncompressed kernel image.
 * Work around this by resetting the ethernet MAC before we uncompress.
 */
#define PHYS_ETH_SELF_CTL		0x80010020
#define ETH_SELF_CTL_RESET		0x00000001

static void ethernet_reset(void)
{
	unsigned int v;

	/* Reset the ethernet MAC.  */
	v = __raw_readl(PHYS_ETH_SELF_CTL);
	__raw_writel(v | ETH_SELF_CTL_RESET, PHYS_ETH_SELF_CTL);

	/* Wait for reset to finish.  */
	while (__raw_readl(PHYS_ETH_SELF_CTL) & ETH_SELF_CTL_RESET)
		;
}


/*
 * We don't have clock management for the UARTs (amba-pl010)
 * yet, so hackily enable all UART clocks here for now.
 */
#define PHYS_SYSCON_DEVICE_CONFIG	0x80930080
#define PHYS_SYSCON_SWLOCK		0x809300c0

static void enable_all_uart_clocks(void)
{
	unsigned int v;

	v = __raw_readl(PHYS_SYSCON_DEVICE_CONFIG);
	__raw_writel(0xaa, PHYS_SYSCON_SWLOCK);
	__raw_writel(v | 0x01140000, PHYS_SYSCON_DEVICE_CONFIG);
}


/*
 * Some bootloaders don't turn on the UARTBAUD bit, which means that
 * the UARTs will be running off a divided 7.3728 MHz clock instead of
 * the 14.7456 MHz peripheral clock when linux boots.
 *
 * We detect that condition here and fix it by turning on UARTBAUD, and
 * then reprogramming the divisors on all enabled UARTs to twice what
 * they were before we turned UARTBAUD on, to preserve the programmed
 * baud rate.
 */
#define PHYS_SYSCON_CLOCK_CONTROL	0x80930004
#define SYSCON_CLOCK_UARTBAUD		0x20000000
#define PHYS_UART1_BASE			0x808c0000
#define PHYS_UART2_BASE			0x808d0000
#define PHYS_UART3_BASE			0x808e0000

static void uart_divisor_times_two(unsigned int base)
{
	u16 divisor;

	divisor = __raw_readb(base + 0x0c) << 8;
	divisor |= __raw_readb(base + 0x10);
	if (divisor) {
		divisor = (2 * (divisor + 1)) - 1;
		__raw_writeb(divisor >> 8, base + 0x0c);
		__raw_writeb(divisor & 0xff, base + 0x10);
		__raw_writeb(__raw_readb(base + 0x08), base + 0x08);
	}
}

static void fix_uart_base(void)
{
	unsigned int v;

	v = __raw_readl(PHYS_SYSCON_CLOCK_CONTROL);
	if ((v & SYSCON_CLOCK_UARTBAUD) == 0) {
		v |= SYSCON_CLOCK_UARTBAUD;
		__raw_writel(v, PHYS_SYSCON_CLOCK_CONTROL);

		uart_divisor_times_two(PHYS_UART1_BASE);
		uart_divisor_times_two(PHYS_UART2_BASE);
		uart_divisor_times_two(PHYS_UART3_BASE);
	}
}


static void arch_decomp_setup(void)
{
	ethernet_reset();
	enable_all_uart_clocks();
	fix_uart_base();
}

#define arch_decomp_wdog()
